=== modified file 'CMakeLists.txt'
--- CMakeLists.txt	2013-12-05 17:25:17 +0000
+++ CMakeLists.txt	2013-12-11 14:52:03 +0000
@@ -41,6 +41,7 @@
 
 find_package(PkgConfig REQUIRED)
 pkg_check_modules(TP_QT5 REQUIRED TelepathyQt5)
+pkg_check_modules(SQLITE3 REQUIRED sqlite3)
 
 pkg_check_modules(PULSEAUDIO libpulse)
 if (PULSEAUDIO_FOUND)
@@ -60,6 +61,7 @@
 include_directories(${TP_QT5_INCLUDE_DIRS})
 include_directories(${Qt5Core_INCLUDE_DIRS})
 include_directories(${Qt5DBus_INCLUDE_DIRS})
+include_directories(${SQLITE3_INCLUDE_DIRS})
 include_directories(/usr/include/telepathy-qt5/)
 include_directories(/usr/include/ofono-qt/)
 
@@ -67,20 +69,50 @@
 find_library(TELEPATHY_QT5_SERVICE_LIBRARIES telepathy-qt5-service)
 find_library(OFONO_QT_LIBRARIES ofono-qt)
 
+qt5_add_resources(telepathyfono_RES sqlitetelepathyofono.qrc)
+
+# update the .qrc file automatically when there are new schema files
+file(GLOB QRC_RESOURCE_FILES ${CMAKE_CURRENT_SOURCE_DIR}/schema/*.sql ${CMAKE_CURRENT_SOURCE_DIR}/schema/*.info)
+set(QRC_FILE ${CMAKE_CURRENT_SOURCE_DIR}/sqlitetelepathyofono.qrc)
+add_custom_command(
+    OUTPUT ${QRC_FILE}
+    COMMAND ${CMAKE_CURRENT_SOURCE_DIR}/update_qrc.sh ${QRC_FILE}
+    WORKING_DIRECTORY ${CMAKE_CURRENT_SOURCE_DIR}
+    DEPENDS ${QRC_RESOURCE_FILES}
+    )
+add_custom_target(qrc_update DEPENDS ${QRC_FILE} schema_update)
+
+set(TELEPATHY_OFONO_SRC 
+   main.cpp
+   protocol.cpp
+   connection.cpp
+   ofonotextchannel.cpp
+   ofonocallchannel.cpp
+   voicemailiface.cpp
+   speakeriface.cpp
+   mmsdmanager.cpp
+   mmsdservice.cpp
+   mmsdmessage.cpp
+   phoneutils.cpp
+   sqlitedatabase.cpp
+   ${telepathyfono_RES})
+
 if(USE_PULSEAUDIO)
-    add_executable(${TELEPATHY_OFONO} main.cpp protocol.cpp connection.cpp ofonotextchannel.cpp ofonocallchannel.cpp voicemailiface.cpp speakeriface.cpp mmsdmanager.cpp mmsdservice.cpp mmsdmessage.cpp qpulseaudioengine.cpp phoneutils.cpp)
+    add_executable(${TELEPATHY_OFONO} qpulseaudioengine.cpp ${TELEPATHY_OFONO_SRC})
 else(USE_PULSEAUDIO)
-    add_executable(${TELEPATHY_OFONO} main.cpp protocol.cpp connection.cpp ofonotextchannel.cpp ofonocallchannel.cpp voicemailiface.cpp speakeriface.cpp mmsdmanager.cpp mmsdservice.cpp mmsdmessage.cpp phoneutils.cpp)
+    add_executable(${TELEPATHY_OFONO} ${TELEPATHY_OFONO_SRC})
 endif(USE_PULSEAUDIO)
-qt5_use_modules(${TELEPATHY_OFONO} Core DBus)
+qt5_use_modules(${TELEPATHY_OFONO} Core DBus Sql)
+add_dependencies(${TELEPATHY_OFONO} schema_update qrc_update)
 
 enable_testing()
 
-target_link_libraries(${TELEPATHY_OFONO} ${Qt5Core_LIBRARIES} ${Qt5DBus_LIBRARIES} ${WAUDIO_LIBRARIES} -ltelepathy-qt5 ${TELEPATHY_QT5_SERVICE_LIBRARIES} ${OFONO_QT_LIBRARIES} ${PULSEAUDIO_LIBRARIES})
+target_link_libraries(${TELEPATHY_OFONO} ${Qt5Core_LIBRARIES} ${Qt5DBus_LIBRARIES} ${WAUDIO_LIBRARIES} -ltelepathy-qt5 ${TELEPATHY_QT5_SERVICE_LIBRARIES} ${OFONO_QT_LIBRARIES} ${PULSEAUDIO_LIBRARIES} ${SQLITE3_LIBRARIES})
 install(TARGETS ${TELEPATHY_OFONO} DESTINATION ${DAEMON_DIR})
 
 configure_file(ofono.service.in org.freedesktop.Telepathy.ConnectionManager.ofono.service)
 install (FILES ofono.manager DESTINATION share/telepathy/managers)
 install (FILES ${CMAKE_CURRENT_BINARY_DIR}/org.freedesktop.Telepathy.ConnectionManager.ofono.service DESTINATION share/dbus-1/services)
 
+add_subdirectory(schema)
 add_subdirectory(tests)

=== modified file 'connection.cpp'
--- connection.cpp	2013-12-03 17:35:09 +0000
+++ connection.cpp	2013-12-11 14:52:03 +0000
@@ -17,6 +17,9 @@
  */
 
 #include <QDebug>
+#include <QStandardPaths>
+#include <QSqlQuery>
+#include <QSqlError>
 
 #include <TelepathyQt/Constants>
 #include <TelepathyQt/BaseChannel>
@@ -41,6 +44,8 @@
 #include "qpulseaudioengine.h"
 #endif
 
+#include "sqlitedatabase.h"
+
 static void enable_earpiece()
 {
 #ifdef USE_AUDIOFLINGER
@@ -141,7 +146,8 @@
     mHandleCount(0),
     mRegisterTimer(new QTimer(this)),
     mMmsdManager(new MMSDManager(this)),
-    mSpeakerMode(false)
+    mSpeakerMode(false),
+    mDatabase(SQLiteDatabase::instance()->database())
 {
     setSelfHandle(newHandle("<SelfHandle>"));
 
@@ -227,6 +233,7 @@
     plugInterface(Tp::AbstractConnectionInterfacePtr::dynamicCast(contactsIface));
 
     QObject::connect(mOfonoMessageManager, SIGNAL(incomingMessage(QString,QVariantMap)), this, SLOT(onOfonoIncomingMessage(QString,QVariantMap)));
+    QObject::connect(mOfonoMessageManager, SIGNAL(statusReport(QString,QVariantMap)), this, SLOT(onDeliveryReportReceived(QString,QVariantMap)));
     QObject::connect(mOfonoVoiceCallManager, SIGNAL(callAdded(QString,QVariantMap)), SLOT(onOfonoCallAdded(QString, QVariantMap)));
     QObject::connect(mOfonoVoiceCallManager, SIGNAL(validityChanged(bool)), SLOT(onValidityChanged(bool)));
     QObject::connect(mOfonoNetworkRegistration, SIGNAL(statusChanged(QString)), SLOT(onOfonoNetworkRegistrationChanged(QString)));
@@ -245,6 +252,64 @@
     // workaround: we can't add services here as tp-ofono interfaces are not exposed on dbus
     // todo: use QDBusServiceWatcher
     QTimer::singleShot(1000, this, SLOT(onCheckMMSServices()));
+
+    populatePendingMessages();
+}
+
+void oFonoConnection::populatePendingMessages()
+{
+    QSqlQuery query(SQLiteDatabase::instance()->database());
+    QString queryString("SELECT messageId,recipientId,timestamp FROM pending_messages");
+    query.prepare(queryString);
+    if (!query.exec()) {
+        qCritical() << "Error:" << query.lastError() << query.lastQuery();
+        return;
+    }
+
+    while (query.next()) {
+        PendingMessage message;
+        QString messageId = query.value(0).toString();
+        message.recipientId = query.value(1).toString();
+        message.timestamp = QDateTime::fromString(query.value(2).toString(), Qt::ISODate);
+        mPendingMessages[messageId] = message;
+    }
+}
+
+void oFonoConnection::addPendingMessage(const QString &messageId, const QString &recipientId)
+{
+    QSqlQuery query(SQLiteDatabase::instance()->database());
+
+    PendingMessage message;
+    message.recipientId = recipientId;
+    message.timestamp = QDateTime::currentDateTimeUtc();
+
+    QString queryString("INSERT into pending_messages (messageId, recipientId, timestamp) VALUES (:messageId, :recipientId, :timestamp)");
+    query.prepare(queryString);
+    query.bindValue(":messageId", messageId);
+    query.bindValue(":recipientId", message.recipientId);
+    query.bindValue(":timestamp", message.timestamp.toString(Qt::ISODate));
+
+    if (!query.exec()) {
+        qCritical() << "Error:" << query.lastError() << query.lastQuery();
+        return;
+    }
+    mPendingMessages[messageId] = message;
+}
+
+void oFonoConnection::removePendingMessage(const QString &messageId)
+{
+    QSqlQuery query(SQLiteDatabase::instance()->database());
+    QString queryString("DELETE FROM pending_messages WHERE messageId=:messageId");
+
+    query.prepare(queryString);
+    query.bindValue(":messageId", messageId);
+
+    if (!query.exec()) {
+        qCritical() << "Error:" << query.lastError() << query.lastQuery();
+        return;
+    }
+
+    mPendingMessages.remove(messageId);
 }
 
 void oFonoConnection::onCheckMMSServices()
@@ -653,6 +718,32 @@
     return mOfonoCallVolume;
 }
 
+void oFonoConnection::onDeliveryReportReceived(const QString &messageId, const QVariantMap& info)
+{
+    if (!mPendingMessages.contains(messageId)) {
+        return;
+    }
+    removePendingMessage(messageId);
+    const QString normalizedNumber = PhoneUtils::normalizePhoneNumber(mPendingMessages[messageId].recipientId);
+    // check if there is an open channel for this sender and use it
+    Q_FOREACH(const QString &phoneNumber, mTextChannels.keys()) {
+        if (PhoneUtils::comparePhoneNumbers(normalizedNumber, phoneNumber)) {
+            mTextChannels[phoneNumber]->deliveryReportReceived(messageId, info["Delivered"].toBool());
+            return;
+        }
+    }
+
+    Tp::DBusError error;
+    bool yours;
+    uint handle = newHandle(normalizedNumber);
+    ensureChannel(TP_QT_IFACE_CHANNEL_TYPE_TEXT,Tp::HandleTypeContact, handle, yours, handle, false, &error);
+    if(error.isValid()) {
+        qWarning() << "Error creating channel for incoming message" << error.name() << error.message();
+        return;
+    }
+    mTextChannels[normalizedNumber]->deliveryReportReceived(messageId, info["Delivered"].toBool());
+}
+
 void oFonoConnection::onOfonoIncomingMessage(const QString &message, const QVariantMap &info)
 {
     const QString normalizedNumber = PhoneUtils::normalizePhoneNumber(info["Sender"].toString());

=== modified file 'connection.h'
--- connection.h	2013-11-08 14:53:53 +0000
+++ connection.h	2013-12-11 14:52:03 +0000
@@ -21,6 +21,7 @@
 
 // qt
 #include <QTimer>
+#include <QSqlDatabase>
 
 // telepathy-qt
 #include <TelepathyQt/BaseConnection>
@@ -52,6 +53,12 @@
 class oFonoCallChannel;
 class MMSDService;
 
+struct PendingMessage
+{
+    QString recipientId;
+    QDateTime timestamp;
+};
+
 class oFonoConnection : public Tp::BaseConnection
 {
     Q_OBJECT
@@ -93,6 +100,10 @@
                                          uint targetHandle, Tp::DBusError *error);
 
     ~oFonoConnection();
+    void addPendingMessage(const QString &objectPath, const QString &id);
+    void removePendingMessage(const QString &objectPath);
+    void populatePendingMessages();
+
 Q_SIGNALS:
     void speakerModeChanged(bool active);
 
@@ -114,6 +125,7 @@
     void onMMSPropertyChanged(QString property, QVariant value);
     void onCheckMMSServices();
     void onMessageRead(const QString &id);
+    void onDeliveryReportReceived(const QString &messageId, const QVariantMap &info);
 
 private:
     bool isNetworkRegistered();
@@ -138,6 +150,8 @@
     QMap<QString, MMSDService*> mMmsdServices;
     QMap<QString, QList<MMSDMessage*> > mServiceMMSList;
     bool mSpeakerMode;
+    QSqlDatabase mDatabase;
+    QHash<QString, PendingMessage> mPendingMessages;
 };
 
 #endif

=== modified file 'debian/control'
--- debian/control	2013-12-05 19:10:23 +0000
+++ debian/control	2013-12-11 14:52:03 +0000
@@ -8,11 +8,14 @@
                dconf-cli,
                gnome-keyring,
                libofono-qt-dev (>= 1.5),
+               libqt5sql5-sqlite,
                libtelepathy-qt5-dev (>= 0.9.3),
                libpulse-dev [armhf],
+               libsqlite3-dev,
                pulseaudio,
                qt5-default (>= 5.0),
                qtbase5-dev (>= 5.0),
+               sqlite3,
                telepathy-mission-control-5
 Standards-Version: 3.9.4
 Homepage: https://launchpad.net/telepathy-ofono

=== modified file 'ofonotextchannel.cpp'
--- ofonotextchannel.cpp	2013-08-20 21:24:09 +0000
+++ ofonotextchannel.cpp	2013-12-11 14:52:03 +0000
@@ -86,6 +86,24 @@
     return mBaseChannel;
 }
 
+void oFonoTextChannel::sendDeliveryReport(const QString &messageId, Tp::DeliveryStatus status)
+{
+    Tp::MessagePartList partList;
+    Tp::MessagePart header;
+    header["message-sender"] = QDBusVariant(mTargetHandle);
+    header["message-sender-id"] = QDBusVariant(mPhoneNumber);
+    header["message-type"] = QDBusVariant(Tp::ChannelTextMessageTypeDeliveryReport);
+    header["delivery-status"] = QDBusVariant(status);
+    header["delivery-token"] = QDBusVariant(messageId);
+    partList << header;
+    mTextChannel->addReceivedMessage(partList);
+}
+
+void oFonoTextChannel::deliveryReportReceived(const QString &messageId, bool success)
+{
+    sendDeliveryReport(messageId, success ? Tp::DeliveryStatusDelivered : Tp::DeliveryStatusPermanentlyFailed);
+}
+
 void oFonoTextChannel::messageAcknowledged(const QString &id)
 {
     Q_EMIT messageRead(id);
@@ -105,6 +123,7 @@
             error->set(TP_QT_ERROR_INVALID_ARGUMENT, mConnection->messageManager()->errorMessage());
         }
     }
+    mConnection->addPendingMessage(objpath, mPhoneNumber);
 
     OfonoMessage *msg = new OfonoMessage(objpath);
     QObject::connect(msg, SIGNAL(stateChanged(QString)), SLOT(onOfonoMessageStateChanged(QString)));
@@ -117,26 +136,19 @@
     if(msg) {
         Tp::DeliveryStatus delivery_status;
         if (status == "sent") {
-            delivery_status = Tp::DeliveryStatusDelivered;
+            delivery_status = Tp::DeliveryStatusAccepted;
             msg->deleteLater();
         } else if(status == "failed") {
             delivery_status = Tp::DeliveryStatusPermanentlyFailed;
+            mConnection->removePendingMessage(msg->path());
             msg->deleteLater();
         } else if(status == "pending") {
-            delivery_status = Tp::DeliveryStatusAccepted;
+            delivery_status = Tp::DeliveryStatusTemporarilyFailed;
         } else {
             delivery_status = Tp::DeliveryStatusUnknown;
         }
 
-        Tp::MessagePartList partList;
-        Tp::MessagePart header;
-        header["message-sender"] = QDBusVariant(mTargetHandle);
-        header["message-sender-id"] = QDBusVariant(mPhoneNumber);
-        header["message-type"] = QDBusVariant(Tp::ChannelTextMessageTypeDeliveryReport);
-        header["delivery-status"] = QDBusVariant(delivery_status);
-        header["delivery-token"] = QDBusVariant(msg->path());
-        partList << header;
-        mTextChannel->addReceivedMessage(partList);
+        sendDeliveryReport(msg->path(), delivery_status);
     }
 }
 

=== modified file 'ofonotextchannel.h'
--- ofonotextchannel.h	2013-07-08 15:55:12 +0000
+++ ofonotextchannel.h	2013-12-11 14:52:03 +0000
@@ -40,6 +40,7 @@
     Tp::BaseChannelPtr baseChannel();
     void messageAcknowledged(const QString &id);
     void mmsReceived(const QString &id, const QVariantMap &properties);
+    void deliveryReportReceived(const QString& messageId, bool success);
 
 private Q_SLOTS:
     void onOfonoMessageStateChanged(QString status);
@@ -48,6 +49,7 @@
     void messageRead(const QString &id);
 
 private:
+    void sendDeliveryReport(const QString &messageId, Tp::DeliveryStatus status);
     ~oFonoTextChannel();
     Tp::BaseChannelPtr mBaseChannel;
     QString mPhoneNumber;

=== added directory 'schema'
=== added file 'schema/CMakeLists.txt'
--- schema/CMakeLists.txt	1970-01-01 00:00:00 +0000
+++ schema/CMakeLists.txt	2013-12-11 14:52:03 +0000
@@ -0,0 +1,13 @@
+file(GLOB SCHEMA_FILES ${CMAKE_CURRENT_SOURCE_DIR}/v*.sql)
+
+set(SCHEMA_FILE ${CMAKE_CURRENT_SOURCE_DIR}/schema.sql)
+set(VERSION_FILE ${CMAKE_CURRENT_SOURCE_DIR}/version.info)
+
+add_custom_command(
+    OUTPUT ${SCHEMA_FILE} ${VERSION_FILE}
+    COMMAND ${CMAKE_CURRENT_SOURCE_DIR}/update_schema.sh ${CMAKE_CURRENT_SOURCE_DIR} ${SCHEMA_FILE} ${VERSION_FILE}
+    WORKING DIRECTORY ${CMAKE_CURRENT_SOURCE_DIR}
+    DEPENDS ${SCHEMA_FILES}
+    )
+
+add_custom_target(schema_update DEPENDS ${SCHEMA_FILE} ${VERSION_FILE})

=== added file 'schema/update_schema.sh'
--- schema/update_schema.sh	1970-01-01 00:00:00 +0000
+++ schema/update_schema.sh	2013-12-11 14:52:03 +0000
@@ -0,0 +1,25 @@
+#!/bin/sh
+
+if [ $# -lt 3 ]; then
+    echo "Usage: $0 <source directory> <target file> <version info file>"
+fi
+
+SOURCE_DIR=$1
+TARGET_FILE=$2
+VERSION_FILE=$3
+
+VERSION="1"
+LATEST_VERSION="1"
+
+TMPFILE=`tempfile`
+
+SCHEMA_FILE="$SOURCE_DIR/v${VERSION}.sql"
+while [ -e $SCHEMA_FILE ]; do
+    cat $SCHEMA_FILE >> $TMPFILE
+    LATEST_VERSION=$VERSION
+    VERSION=$(($VERSION+1))
+    SCHEMA_FILE="$SOURCE_DIR/v${VERSION}.sql"
+done
+
+sqlite3 -init $TMPFILE :memory: .schema > $TARGET_FILE
+echo $LATEST_VERSION > $VERSION_FILE

=== added file 'schema/v1.sql'
--- schema/v1.sql	1970-01-01 00:00:00 +0000
+++ schema/v1.sql	2013-12-11 14:52:03 +0000
@@ -0,0 +1,9 @@
+CREATE TABLE schema_version (
+    version int
+);
+
+CREATE TABLE pending_messages (
+    messageId varchar(255),
+    recipientId varchar(255),
+    timestamp datetime
+);

=== added file 'sqlitedatabase.cpp'
--- sqlitedatabase.cpp	1970-01-01 00:00:00 +0000
+++ sqlitedatabase.cpp	2013-12-11 14:52:03 +0000
@@ -0,0 +1,240 @@
+/*
+ * Copyright (C) 2013 Canonical, Ltd.
+ *
+ * Authors:
+ *  Gustavo Pichorim Boiko <gustavo.boiko@canonical.com>
+ *
+ * This file is part of telepathy-ofono.
+ *
+ * telepathy-ofono is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; version 3.
+ *
+ * telepathy-ofono is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
+#include "phoneutils_p.h"
+#include "sqlite3.h"
+#include "sqlitedatabase.h"
+#include <QStandardPaths>
+#include <QSqlDriver>
+#include <QSqlQuery>
+#include <QSqlError>
+#include <QDebug>
+#include <QFile>
+#include <QDir>
+
+Q_DECLARE_OPAQUE_POINTER(sqlite3*)
+Q_DECLARE_METATYPE(sqlite3*)
+
+// custom sqlite function "comparePhoneNumbers" used to compare IDs if necessary
+void comparePhoneNumbers(sqlite3_context *context, int argc, sqlite3_value **argv)
+{
+    QString arg1((const char*)sqlite3_value_text(argv[0]));
+    QString arg2((const char*)sqlite3_value_text(argv[1]));
+    sqlite3_result_int(context, (int)PhoneUtils::comparePhoneNumbers(arg1, arg2));
+}
+
+SQLiteDatabase::SQLiteDatabase(QObject *parent) :
+    QObject(parent), mSchemaVersion(0)
+{
+    initializeDatabase();
+}
+
+SQLiteDatabase *SQLiteDatabase::instance()
+{
+    static SQLiteDatabase *self = new SQLiteDatabase();
+    return self;
+}
+
+bool SQLiteDatabase::initializeDatabase()
+{
+    mDatabasePath = qgetenv("TP_OFONO_SQLITE_DBPATH");
+
+    if (mDatabasePath.isEmpty()) {
+        mDatabasePath = QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation);
+
+        QDir dir(mDatabasePath);
+        if (!dir.exists("telepathy-ofono") && !dir.mkpath("telepathy-ofono")) {
+            qCritical() << "Failed to create dir";
+            return false;
+        }
+        dir.cd("telepathy-ofono");
+
+        mDatabasePath = dir.absoluteFilePath("telepathy-ofono.sqlite");
+    }
+
+    mDatabase = QSqlDatabase::addDatabase("QSQLITE");
+    mDatabase.setDatabaseName(mDatabasePath);
+
+    // always run the createDatabase function at least during the development
+    if (!createOrUpdateDatabase()) {
+        qCritical() << "Failed to create or update the database";
+        return false;
+    }
+
+    return true;
+}
+
+QSqlDatabase SQLiteDatabase::database() const
+{
+    return mDatabase;
+}
+
+bool SQLiteDatabase::beginTransation()
+{
+    return mDatabase.transaction();
+}
+
+bool SQLiteDatabase::finishTransaction()
+{
+    return mDatabase.commit();
+}
+
+bool SQLiteDatabase::rollbackTransaction()
+{
+    return mDatabase.rollback();
+}
+
+/// this method is to be used mainly by unit tests in order to clean up the database between
+/// tests.
+bool SQLiteDatabase::reopen()
+{
+    mDatabase.close();
+    mDatabase.open();
+
+    // make sure the database is up-to-date after reopening.
+    // this is mainly required for the memory backend used for testing
+    createOrUpdateDatabase();
+}
+
+bool SQLiteDatabase::createOrUpdateDatabase()
+{
+    bool create = !QFile(mDatabasePath).exists();
+
+    if (!mDatabase.open()) {
+        return false;
+    }
+
+    // create the comparePhoneNumbers custom sqlite function
+    sqlite3 *handle = database().driver()->handle().value<sqlite3*>();
+    sqlite3_create_function(handle, "comparePhoneNumbers", 2, SQLITE_ANY, NULL, &comparePhoneNumbers, NULL, NULL);
+
+    parseVersionInfo();
+
+    QSqlQuery query(mDatabase);
+
+    QStringList statements;
+
+    if (create) {
+         statements = parseSchemaFile(":/database/schema/schema.sql");
+    } else {
+        // if the database already exists, we don´t need to create the tables
+        // only check if an update is needed
+        query.exec("SELECT * FROM schema_version");
+        if (!query.exec() || !query.next()) {
+            return false;
+        }
+
+        int upgradeToVersion = query.value(0).toInt() + 1;
+        while (upgradeToVersion <= mSchemaVersion) {
+            statements += parseSchemaFile(QString(":/database/schema/v%1.sql").arg(QString::number(upgradeToVersion)));
+            ++upgradeToVersion;
+        }
+    }
+
+    // if at this point needsUpdate is still false, it means the database is up-to-date
+    if (statements.isEmpty()) {
+        return true;
+    }
+
+    beginTransation();
+
+    Q_FOREACH(const QString &statement, statements) {
+        if (!query.exec(statement)) {
+            qCritical() << "Failed to create or update database. SQL Statements:" << query.lastQuery() << "Error:" << query.lastError();
+            rollbackTransaction();
+            return false;
+        }
+    }
+
+    // now set the new database schema version
+    if (!query.exec("DELETE FROM schema_version")) {
+        qCritical() << "Failed to remove previous schema versions. SQL Statement:" << query.lastQuery() << "Error:" << query.lastError();
+        rollbackTransaction();
+        return false;
+    }
+
+    if (!query.exec(QString("INSERT INTO schema_version VALUES (%1)").arg(mSchemaVersion))) {
+        qCritical() << "Failed to insert new schema version. SQL Statement:" << query.lastQuery() << "Error:" << query.lastError();
+        rollbackTransaction();
+        return false;
+    }
+
+    finishTransaction();
+
+    return true;
+}
+
+QStringList SQLiteDatabase::parseSchemaFile(const QString &fileName)
+{
+    QFile schema(fileName);
+    if (!schema.open(QFile::ReadOnly)) {
+        qCritical() << "Failed to open " << fileName;
+        return QStringList();
+    }
+
+    bool parsingBlock = false;
+    QString statement;
+    QStringList statements;
+
+    // FIXME: this parser is very basic, it needs to be improved in the future
+    //        it does a lot of assumptions based on the structure of the schema.sql file
+
+    QTextStream stream(&schema);
+    while (!stream.atEnd()) {
+        QString line = stream.readLine();
+        bool statementEnded = false;
+
+        statement += line;
+
+        // check if we are parsing a trigger command
+        if (line.trimmed().startsWith("CREATE TRIGGER", Qt::CaseInsensitive)) {
+            parsingBlock = true;
+        } else if (parsingBlock) {
+            if (line.contains("END;")) {
+                parsingBlock = false;
+                statementEnded = true;
+            }
+        } else if (statement.contains(";")) {
+            statementEnded = true;
+        }
+
+        statement += "\n";
+
+        if (statementEnded) {
+            statements.append(statement);
+            statement.clear();
+        }
+    }
+
+    return statements;
+}
+
+void SQLiteDatabase::parseVersionInfo()
+{
+    QFile schema(":/database/schema/version.info");
+    if (!schema.open(QFile::ReadOnly)) {
+        qDebug() << schema.error();
+        qCritical() << "Failed to get database version";
+    }
+
+    QString version = schema.readAll();
+    mSchemaVersion = version.toInt();
+}

=== added file 'sqlitedatabase.h'
--- sqlitedatabase.h	1970-01-01 00:00:00 +0000
+++ sqlitedatabase.h	2013-12-11 14:52:03 +0000
@@ -0,0 +1,56 @@
+/*
+ * Copyright (C) 2013 Canonical, Ltd.
+ *
+ * Authors:
+ *  Gustavo Pichorim Boiko <gustavo.boiko@canonical.com>
+ *
+ * This file is part of telepathy-ofono.
+ *
+ * telepathy-ofono is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; version 3.
+ *
+ * telepathy-ofono is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
+#ifndef SQLITEDATABASE_H
+#define SQLITEDATABASE_H
+
+#include <QObject>
+#include <QSqlDatabase>
+
+class SQLiteDatabase : public QObject
+{
+    Q_OBJECT
+public:
+    static SQLiteDatabase *instance();
+
+    bool initializeDatabase();
+    QSqlDatabase database() const;
+
+    bool beginTransation();
+    bool finishTransaction();
+    bool rollbackTransaction();
+
+    bool reopen();
+
+protected:
+    bool createOrUpdateDatabase();
+    QStringList parseSchemaFile(const QString &fileName);
+    void parseVersionInfo();
+
+private:
+    explicit SQLiteDatabase(QObject *parent = 0);
+    QString mDatabasePath;
+    QSqlDatabase mDatabase;
+    int mSchemaVersion;
+    
+};
+
+#endif // SQLITEDATABASE_H

=== modified file 'tests/CMakeLists.txt'
--- tests/CMakeLists.txt	2013-12-05 15:51:10 +0000
+++ tests/CMakeLists.txt	2013-12-11 14:52:03 +0000
@@ -18,6 +18,7 @@
     set(TMPDIR "/tmp/tpofono_test_home")
     set(TEST_ENVIRONMENT "HOME=${TMPDIR};
                           HISTORY_SQLITE_DBPATH=:memory:;
+                          TP_OFONO_SQLITE_DBPATH=:memory:;
                           XDG_CONFIG_HOME=${TMPDIR};
                           XDG_DATA_HOME=${TMPDIR};
                           XDG_CACHE_DIR=${TMPDIR};

=== added file 'update_qrc.sh'
--- update_qrc.sh	1970-01-01 00:00:00 +0000
+++ update_qrc.sh	2013-12-11 14:52:03 +0000
@@ -0,0 +1,19 @@
+#!/bin/sh
+QRC_FILE=$1
+FILES=`ls schema/*.sql schema/*.info`
+
+# clear the file
+if [ -e $QRC_FILE ]; then
+    rm -f $QRC_FILE
+fi
+
+# and print the contents
+echo '<RCC>' >> $QRC_FILE
+echo '    <qresource prefix="/database">' >> $QRC_FILE
+
+for file in $FILES; do
+    echo "        <file>$file</file>" >> $QRC_FILE
+done
+
+echo '    </qresource>' >> $QRC_FILE
+echo '</RCC>' >> $QRC_FILE

